"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _doubleApi2 = require("../doubleApi");

var _doubleApi3 = _interopRequireDefault(_doubleApi2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var msgTypes = {
    "WELCOME": 0,
    "PREFIX": 1,
    "CALL": 2,
    "CALLRESULT": 3,
    "CALLERROR": 4,
    "SUBSCRIBE": 5,
    "UNSUBSCRIBE": 6,
    "PUBLISH": 7,
    "EVENT": 8,
    "SUBSCRIBED": 9,
    "SUBSCRIBEERROR": 10,
    "HB": 20
};

var wsStates = {
    "CONNECTING": 0,
    "OPEN": 1,
    "CLOSING": 2,
    "CLOSED": 3
};

var helpers = {
    getRandom: function getRandom(min, max) {
        return Math.random() * (max - min) + min;
    }
};

var WSClient = function () {
    function WSClient() {
        var heartBeat = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
        var stringMsgTypes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

        _classCallCheck(this, WSClient);

        this._wsClient = null;
        this._heartBeat = heartBeat;
        this._stringMsgTypes = stringMsgTypes;
        this._callSequence = 0;
        this._heartBeatInterval = 5 * 1000;
        //Public API
        this.open = this.connect = this.connect.bind(this);
        this.close = this.close.bind(this);
        this.call = this.call.bind(this);
        this.subscribe = this.subscribe.bind(this);
        this.unsubscribe = this.unsubscribe.bind(this);
        this.publish = this.publish.bind(this);
        this.event = this.event.bind(this);
        this.registerRpcHandler = this.registerRpcHandler.bind(this);
        //Public event handlers
        this.onclose = null;
        this.onopen = null;
        this.onerror = null;

        Object.defineProperty(this, "state", {
            get: function get() {
                return this._wsClient && this._wsClient.readyState;
            },
            enumerable: true
        });
    }

    _createClass(WSClient, [{
        key: "__resetHandlers",
        value: function __resetHandlers() {
            //Outbound subscriptions (from THIS to SERVER)
            this._eventHandlers = {};
            this._subscribedHandlers = {};
            this._subscribeErrorHandlers = {};
            //Inbound subscriptions (from SERVER to THIS)
            this._subUris = {};
            this._callResponseHandlers = {};
            this._callRequestHandlers = {};
            this._heartBeatHandlers = {};
            //Outbound RPC
            this._callResponseHandlers = {};
            //Inbound RPC
            this._callRequestHandlers = {};
            //HB
            this._heartBeatHandlers = {};
        }
    }, {
        key: "__resetWS",
        value: function __resetWS() {
            clearInterval(this._hbInterval);
            if (this._wsClient) {
                this._wsClient.onopen = null;
                this._wsClient.onclose = null;
                this._wsClient.onmessage = null;
                this._wsClient.onerror = null;
                this._wsClient.close();
            }
        }

        /**
         *
         * @param serverUrl - адрес сервера
         * @param cb - callback, который отработает при успешном соединении с сервером
         * @private
         */

    }, {
        key: "connect",
        value: function connect(serverUrl, cb) {
            this.__resetHandlers();
            this.__resetWS();

            if (!/^(wss?:\/\/).+/.test(serverUrl)) {
                throw new Error("Incorrect server url: " + serverUrl);
            }
            var Client = typeof WebSocket === "undefined" ? this.WebSocket : WebSocket;
            this._serverUrl = serverUrl;
            this._wsClient = new Client(serverUrl);
            this._wsClient.onopen = this._wsOpenedHandler.bind(this);
            this._wsClient.onclose = this._wsClosedHandler.bind(this);
            this._wsClient.onmessage = this._wsMessageHandler.bind(this);
            this._wsClient.onerror = this._wsErrorHandler.bind(this);
            this._connectHandler = cb;
        }
    }, {
        key: "close",
        value: function close() {
            clearTimeout(this._reconnectTimer);
            this.__resetWS();
        }

        /**
         * Remote procedure call
         * @param url
         * @param callback - callback, который вызовется, когда придет ответ с сервера
         * @private
         */

    }, {
        key: "call",
        value: function call(url) {
            if (!url) {
                throw new Error("invalid args: url");
            }
            if (this._wsClient.readyState !== wsStates.OPEN) {
                throw new Error("WebSocket not connected");
            }
            var cb = void 0,
                promise = void 0,
                data = Array.prototype.slice.call(arguments, 1);
            if (typeof data[data.length - 1] === "function") {
                cb = data.pop();
            }

            var _doubleApi = (0, _doubleApi3.default)(cb);

            promise = _doubleApi.promise;
            cb = _doubleApi.cb;


            var callId = ++this._callSequence;
            this._callResponseHandlers[callId] = cb;
            var callData = [this._stringMsgTypes ? "CALL" : msgTypes.CALL, callId, url];
            callData = callData.concat(data);
            this._wsClient.send(JSON.stringify(callData));

            return promise;
        }

        /**
         * Server events subscription
         * @param url
         * @param callback
         * @private
         */

    }, {
        key: "subscribe",
        value: function subscribe(uri, eventCb, okCb, errorCb, params) {
            if (this._wsClient.readyState === wsStates.OPEN) {
                this._eventHandlers[uri] = eventCb;
                this._subscribedHandlers[uri] = okCb;
                this._subscribeErrorHandlers[uri] = errorCb;
                var msg = [this._stringMsgTypes ? "SUBSCRIBE" : msgTypes.SUBSCRIBE, uri];
                if (params) {
                    msg.push(params);
                }
                this._wsClient.send(JSON.stringify(msg));
            } else {
                throw new Error("WebSocket not connected");
            }
        }

        /**
         * Отписка от серверных событий
         * @param url
         * @private
         */

    }, {
        key: "unsubscribe",
        value: function unsubscribe(uri) {
            delete this._eventHandlers[uri];
            delete this._subscribedHandlers[uri];
            delete this._subscribeErrorHandlers[uri];
            if (this._wsClient.readyState === wsStates.OPEN) {
                this._wsClient.send(JSON.stringify([this._stringMsgTypes ? "UNSUBSCRIBE" : msgTypes.UNSUBSCRIBE, uri]));
            }
        }
    }, {
        key: "publish",
        value: function publish(uri, event) {
            if (this._wsClient.readyState === wsStates.OPEN) {
                this._wsClient.send(JSON.stringify([this._stringMsgTypes ? "PUBLISH" : msgTypes.PUBLISH, uri, event]));
            } else {
                throw new Error("WebSocket not connected");
            }
        }
    }, {
        key: "event",
        value: function event(uri, _event) {
            if (this._wsClient.readyState === wsStates.OPEN) {
                if (this._subUris[uri]) {
                    this._wsClient.send(JSON.stringify([this._stringMsgTypes ? "EVENT" : msgTypes.EVENT, uri, _event]));
                } else {
                    console.info("No subscribers for \"" + uri + "\"");
                }
            } else {
                throw new Error("WebSocket not connected");
            }
        }
    }, {
        key: "registerRpcHandler",
        value: function registerRpcHandler(uri, cb) {
            if (typeof uri !== "string" || !uri) {
                throw new Error("Invalid uri, must be non empty string");
            }
            if (typeof cb === "function") {
                this._callRequestHandlers[uri] = cb;
            } else {
                if (cb == null) {
                    delete this._callRequestHandlers[uri];
                } else {
                    throw new Error("Invalid callback, must be function or null");
                }
            }
        }
    }, {
        key: "_wsMessageHandler",
        value: function _wsMessageHandler(rawMsg) {
            var msg = void 0;
            try {
                msg = JSON.parse(rawMsg.data);
            } catch (e) {
                console.error("WSClient: Invalid message JSON");
                return;
            }
            var msgType = msg[0],
                msgId = msg[1],
                msgData = msg.length > 2 ? msg[2] : null;
            if (typeof msgType === "string" && msgTypes.hasOwnProperty(msgType)) {
                msgType = msgTypes[msgType];
            }
            switch (msgType) {
                case msgTypes.EVENT:
                    if (typeof this._eventHandlers[msgId] === "function") {
                        this._eventHandlers[msgId](msgData);
                    }
                    break;
                case msgTypes.SUBSCRIBE:
                    this._subUris[msgId] = true;
                    break;
                case msgTypes.UNSUBSCRIBE:
                    delete this._subUris[msgId];
                    break;
                case msgTypes.SUBSCRIBED:
                    if (typeof this._subscribedHandlers[msgId] === "function") {
                        this._subscribedHandlers[msgId](msgData);
                    }
                    delete this._subscribedHandlers[msgId];
                    break;
                case msgTypes.SUBSCRIBEERROR:
                    if (typeof this._subscribeErrorHandlers[msgId] === "function") {
                        this._subscribeErrorHandlers[msgId](msgData);
                    }
                    delete this._subscribeErrorHandlers[msgId];
                    break;
                case msgTypes.CALL:
                    {
                        var rpcUri = msgData;
                        var _iteratorNormalCompletion = true;
                        var _didIteratorError = false;
                        var _iteratorError = undefined;

                        try {
                            for (var _iterator = Object.keys(this._callRequestHandlers)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                                var uri = _step.value;

                                if (uri === rpcUri || new RegExp(uri).test(rpcUri)) {
                                    this._callRequestHandlers[uri].apply(this, msg.slice(3));
                                }
                            }
                        } catch (err) {
                            _didIteratorError = true;
                            _iteratorError = err;
                        } finally {
                            try {
                                if (!_iteratorNormalCompletion && _iterator.return) {
                                    _iterator.return();
                                }
                            } finally {
                                if (_didIteratorError) {
                                    throw _iteratorError;
                                }
                            }
                        }
                    }
                    break;
                case msgTypes.CALLRESULT:
                    if (typeof this._callResponseHandlers[msgId] === "function") {
                        this._callResponseHandlers[msgId](null, msgData);
                    }
                    delete this._callResponseHandlers[msgId];
                    break;
                case msgTypes.CALLERROR:
                    var err = {
                        desc: msgData,
                        details: msg.length > 3 ? msg[3] : null
                    };
                    if (typeof this._callResponseHandlers[msgId] === "function") {
                        this._callResponseHandlers[msgId](err, null);
                    }
                    delete this._callResponseHandlers[msgId];
                    break;
                case msgTypes.HB:
                    if (typeof this._heartBeatHandlers[msgId] === "function") {
                        this._heartBeatHandlers[msgId](msgData);
                    }
                    delete this._heartBeatHandlers[msgId];
                    break;
            }
        }
    }, {
        key: "_wsOpenedHandler",
        value: function _wsOpenedHandler(e) {
            if (this._heartBeat) {
                this._startHeartbeat.call(this);
            }
            if (typeof this._connectHandler === "function") {
                this._connectHandler();
            }
            if (typeof this.onopen === "function") {
                this.onopen(e);
            }
        }
    }, {
        key: "_wsClosedHandler",
        value: function _wsClosedHandler(closeEvent) {
            clearInterval(this._hbInterval);
            this._reconnectTimer = setTimeout(this._startReconnect.bind(this), helpers.getRandom(4, 8) * 1000);

            if (typeof this.onclose === "function") {
                this.onclose(closeEvent);
            }
        }
    }, {
        key: "_wsErrorHandler",
        value: function _wsErrorHandler(err) {
            if (typeof this.onerror === "function") {
                this.onerror(err);
            } else {
                console.error(err);
            }
        }
    }, {
        key: "_startReconnect",
        value: function _startReconnect() {
            if (this._wsClient && this._wsClient.readyState === wsStates.CLOSED) {
                this.connect(this._serverUrl);
            }
        }
    }, {
        key: "_startHeartbeat",
        value: function _startHeartbeat() {
            var _this = this;

            var hbCount = 0,
                hbCounter = 0;
            this._hbInterval = setInterval(function () {
                if (!_this._wsClient || _this._wsClient.readyState !== wsStates.OPEN) {
                    clearInterval(_this._hbInterval);
                    return;
                }
                _this._sendHeartbeat.call(_this, hbCount++, function () {
                    hbCounter = 0;
                });
                hbCounter++;
                if (hbCounter > 5) {
                    console.warn("Ping timeout, reconnecting...");
                    _this.close();
                    _this.connect(_this._serverUrl);
                }
            }, this._heartBeatInterval);
        }
    }, {
        key: "_sendHeartbeat",
        value: function _sendHeartbeat(hbNumber, cb) {
            this._heartBeatHandlers[hbNumber] = cb;
            this._wsClient.send(JSON.stringify([this._stringMsgTypes ? "HB" : msgTypes.HB, hbNumber]));
        }
    }]);

    return WSClient;
}();

exports.default = WSClient;