"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.decodeToken = decodeToken;
exports.isTokenInvalid = isTokenInvalid;
function decodeToken(token) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (typeof token !== "string") {
        throw new Error("token must be a string");
    }
    var pos = options.header === true ? 0 : 1;
    var base64Data = token.split(".")[pos];
    var encodedData = atob(base64Data).replace(/(.)/g, function (m, p) {
        var code = p.charCodeAt(0).toString(16).toUpperCase();
        if (code.length < 2) {
            code = "0" + code;
        }
        return "%" + code;
    });
    try {
        var stringData = decodeURIComponent(encodedData);
        var data = JSON.parse(stringData);
        return data;
    } catch (e) {
        throw new Error("invalid token specified: " + e.message);
    }
}

function isTokenInvalid(token) {
    var serverUri = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "";

    return fetch(serverUri + "check-jwt", {
        method: "POST",
        headers: {
            Authorization: "Bearer " + token
        }
    }).then(function (response) {
        if (response.status >= 200 && response.status < 300) {
            return response.json();
        }
    }).then(function (data) {
        if (data && data.valid === false) {
            return true;
        }
    });
}